@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 # Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
@@ -9,10 +8,12 @@
 Test utilities.
 """
 
+from __future__ import print_function
 import atexit
 import contextlib
 import errno
 import functools
+import ipaddress  # python >= 3.3 / requires "pip install ipaddress"
 import os
 import re
 import shutil
@@ -28,10 +29,7 @@ import warnings
 from socket import AF_INET
 from socket import SOCK_DGRAM
 from socket import SOCK_STREAM
-try:
-    import ipaddress  # python >= 3.3
-except ImportError:
-    ipaddress = None
+
 try:
     from unittest import mock  # py3
 except ImportError:
@@ -39,7 +37,6 @@ except ImportError:
 
 import psutil
 from psutil import LINUX
-from psutil import OSX
 from psutil import POSIX
 from psutil import WINDOWS
 from psutil._compat import PY3
@@ -63,6 +60,7 @@ if PY3:
 else:
     import imp as importlib
 
+
 __all__ = [
     # constants
     'APPVEYOR', 'DEVNULL', 'GLOBAL_TIMEOUT', 'MEMORY_TOLERANCE', 'NO_RETRIES',
@@ -76,7 +74,7 @@ __all__ = [
     'skip_on_access_denied', 'skip_on_not_implemented', 'retry_before_failing',
     'run_test_module_by_name',
     # fs utils
-    'chdir', 'safe_remove', 'safe_rmdir', 'create_temp_executable_file',
+    'chdir', 'safe_rmpath', 'create_exe',
     # subprocesses
     'pyrun', 'reap_children', 'get_test_subprocess',
     # os
@@ -104,16 +102,19 @@ AF_INET6 = getattr(socket, "AF_INET6")
 AF_UNIX = getattr(socket, "AF_UNIX", None)
 PYTHON = os.path.realpath(sys.executable)
 DEVNULL = open(os.devnull, 'r+')
-TESTFN = os.path.join(os.getcwd(), "$testfile")
-TESTFN_UNICODE = TESTFN + "ƒőő"
-TESTFILE_PREFIX = 'psutil-test-suite-'
-TOX = os.getenv('TOX') or '' in ('1', 'true')
-PYPY = '__pypy__' in sys.builtin_module_names
+
+TESTFILE_PREFIX = '$testfn'
+TESTFN = os.path.join(os.path.realpath(os.getcwd()), TESTFILE_PREFIX)
+_TESTFN = TESTFN + '-internal'
+TESTFN_UNICODE = TESTFN + "-ƒőő"
 if not PY3:
     try:
-        TESTFN_UNICODE = unicode(TESTFN_UNICODE, sys.getfilesystemencoding())
+        TESTFN_UNICODE = unicode(TESTFN, sys.getfilesystemencoding())
     except UnicodeDecodeError:
-        TESTFN_UNICODE = TESTFN + "???"
+        TESTFN_UNICODE = TESTFN + "-???"
+
+TOX = os.getenv('TOX') or '' in ('1', 'true')
+PYPY = '__pypy__' in sys.builtin_module_names
 
 ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                         '..', '..'))
@@ -128,8 +129,6 @@ TRAVIS = bool(os.environ.get('TRAVIS'))
 # (http://www.appveyor.com/)
 APPVEYOR = bool(os.environ.get('APPVEYOR'))
 
-if TRAVIS or 'tox' in sys.argv[0]:
-    import ipaddress
 if TRAVIS or APPVEYOR:
     GLOBAL_TIMEOUT = GLOBAL_TIMEOUT * 4
 VERBOSITY = 1 if os.getenv('SILENT') or TOX else 2
@@ -190,39 +189,26 @@ class ThreadTask(threading.Thread):
 _subprocesses_started = set()
 
 
-def get_test_subprocess(cmd=None, wait=False, **kwds):
+def get_test_subprocess(cmd=None, **kwds):
     """Return a subprocess.Popen object to use in tests.
     By default stdout and stderr are redirected to /dev/null and the
     python interpreter is used as test process.
-    If 'wait' is True attemps to make sure the process is in a
-    reasonably initialized state.
+    It also attemps to make sure the process is in a reasonably
+    initialized state.
     """
-    if cmd is None:
-        pyline = ""
-        if wait:
-            pyline += "open(r'%s', 'w'); " % TESTFN
-        # A process living for 30 secs. We sleep N times (as opposed to
-        # once) in order to be nicer towards Windows which doesn't handle
-        # interrupt signals properly.
-        pyline += "import time; [time.sleep(0.01) for x in range(3000)];"
-        cmd_ = [PYTHON, "-c", pyline]
-    else:
-        cmd_ = cmd
     kwds.setdefault("stdin", DEVNULL)
     kwds.setdefault("stdout", DEVNULL)
-    kwds.setdefault("stderr", DEVNULL)
-    sproc = subprocess.Popen(cmd_, **kwds)
-    if wait:
-        if cmd is None:
-            stop_at = time.time() + 3
-            while stop_at > time.time():
-                if os.path.exists(TESTFN):
-                    break
-                time.sleep(0.001)
-            else:
-                warn("couldn't make sure test file was actually created")
-        else:
-            wait_for_pid(sproc.pid)
+    if cmd is None:
+        safe_rmpath(_TESTFN)
+        pyline = "from time import sleep;"
+        pyline += "open(r'%s', 'w').close();" % _TESTFN
+        pyline += "sleep(60)"
+        cmd = [PYTHON, "-c", pyline]
+        sproc = subprocess.Popen(cmd, **kwds)
+        wait_for_file(_TESTFN, delete_file=True, empty=True)
+    else:
+        sproc = subprocess.Popen(cmd, **kwds)
+        wait_for_pid(sproc.pid)
     _subprocesses_started.add(sproc)
     return sproc
 
@@ -231,7 +217,7 @@ _testfiles = []
 
 
 def pyrun(src):
-    """Run python code 'src' in a separate interpreter.
+    """Run python 'src' code in a separate interpreter.
     Return interpreter subprocess.
     """
     if PY3:
@@ -267,18 +253,23 @@ def sh(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
 
 
 def reap_children(recursive=False):
-    """Kill any subprocess started by this test suite and ensure that
-    no zombies stick around to hog resources and create problems when
-    looking for refleaks.
+    """Terminate and wait() any subprocess started by this test suite
+    and ensure that no zombies stick around to hog resources and
+    create problems  when looking for refleaks.
+
+    If resursive is True it also tries to terminate and wait()
+    all grandchildren started by this process.
     """
-    # Get the children here, before terminating the sub processes
-    # as we don't want to lose the intermediate reference in case
-    # of grand children.
+    # Get the children here, before terminating the children sub
+    # processes as we don't want to lose the intermediate reference
+    # in case of grandchildren.
     if recursive:
         children = psutil.Process().children(recursive=True)
     else:
         children = []
 
+    # Terminate subprocess.Popen instances "cleanly" by closing their
+    # fds and wiat()ing for them in order to avoid zombies.
     subprocs = _subprocesses_started.copy()
     _subprocesses_started.clear()
     for subp in subprocs:
@@ -287,12 +278,23 @@ def reap_children(recursive=False):
         except OSError as err:
             if err.errno != errno.ESRCH:
                 raise
+        if subp.stdout:
+            subp.stdout.close()
+        if subp.stderr:
+            subp.stderr.close()
         try:
-            subp.wait()
-        except OSError as err:
-            if err.errno != errno.ECHILD:
-                raise
+            # Flushing a BufferedWriter may raise an error.
+            if subp.stdin:
+                subp.stdin.close()
+        finally:
+            # Wait for the process to terminate, to avoid zombies.
+            try:
+                subp.wait()
+            except OSError as err:
+                if err.errno != errno.ECHILD:
+                    raise
 
+    # Terminates grandchildren.
     if children:
         for p in children:
             try:
@@ -301,14 +303,15 @@ def reap_children(recursive=False):
                 pass
         gone, alive = psutil.wait_procs(children, timeout=GLOBAL_TIMEOUT)
         for p in alive:
-            warn("couldn't terminate process %s" % p)
+            warn("couldn't terminate process %r; attempting kill()" % p)
             try:
                 p.kill()
             except psutil.NoSuchProcess:
                 pass
-            _, alive = psutil.wait_procs(alive, timeout=GLOBAL_TIMEOUT)
-            if alive:
-                warn("couldn't not kill processes %s" % str(alive))
+        _, alive = psutil.wait_procs(alive, timeout=GLOBAL_TIMEOUT)
+        if alive:
+            for p in alive:
+                warn("process %r survived kill()" % p)
 
 
 # ===================================================================
@@ -370,50 +373,98 @@ else:
 # ===================================================================
 
 
-def wait_for_pid(pid, timeout=GLOBAL_TIMEOUT):
+class retry(object):
+    """A retry decorator."""
+
+    def __init__(self,
+                 exception=Exception,
+                 timeout=None,
+                 retries=None,
+                 interval=0.001,
+                 logfun=lambda s: print(s, file=sys.stderr),
+                 ):
+        if timeout and retries:
+            raise ValueError("timeout and retries args are mutually exclusive")
+        self.exception = exception
+        self.timeout = timeout
+        self.retries = retries
+        self.interval = interval
+        self.logfun = logfun
+
+    def __iter__(self):
+        if self.timeout:
+            stop_at = time.time() + self.timeout
+            while time.time() < stop_at:
+                yield
+        elif self.retries:
+            for _ in range(self.retries):
+                yield
+        else:
+            while True:
+                yield
+
+    def sleep(self):
+        if self.interval is not None:
+            time.sleep(self.interval)
+
+    def __call__(self, fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwargs):
+            exc = None
+            for _ in self:
+                try:
+                    return fun(*args, **kwargs)
+                except self.exception as _:
+                    exc = _
+                    if self.logfun is not None:
+                        self.logfun(exc)
+                    self.sleep()
+            else:
+                if PY3:
+                    raise exc
+                else:
+                    raise
+
+        # This way the user of the decorated function can change config
+        # parameters.
+        wrapper.decorator = self
+        return wrapper
+
+
+@retry(exception=psutil.NoSuchProcess, logfun=None, timeout=GLOBAL_TIMEOUT,
+       interval=0.001)
+def wait_for_pid(pid):
     """Wait for pid to show up in the process list then return.
     Used in the test suite to give time the sub process to initialize.
     """
-    raise_at = time.time() + timeout
-    while True:
-        if pid in psutil.pids():
-            # give it one more iteration to allow full initialization
-            time.sleep(0.01)
-            return
-        time.sleep(0.0001)
-        if time.time() >= raise_at:
-            raise RuntimeError("Timed out")
+    psutil.Process(pid)
+    if WINDOWS:
+        # give it some more time to allow better initialization
+        time.sleep(0.01)
 
 
-def wait_for_file(fname, timeout=GLOBAL_TIMEOUT, delete_file=True):
-    """Wait for a file to be written on disk."""
-    stop_at = time.time() + timeout
-    while time.time() < stop_at:
-        try:
-            with open(fname, "r") as f:
-                data = f.read()
-            if not data:
-                continue
-            if delete_file:
-                os.remove(fname)
-            return data
-        except IOError:
-            time.sleep(0.001)
-    raise RuntimeError(
-        "timed out after %s secs (couldn't read file)" % timeout)
+@retry(exception=(EnvironmentError, AssertionError), logfun=None,
+       timeout=GLOBAL_TIMEOUT, interval=0.001)
+def wait_for_file(fname, delete_file=True, empty=False):
+    """Wait for a file to be written on disk with some content."""
+    with open(fname, "rb") as f:
+        data = f.read()
+    if not empty:
+        assert data
+    if delete_file:
+        os.remove(fname)
+    return data
 
 
-def call_until(fun, expr, timeout=GLOBAL_TIMEOUT):
+@retry(exception=AssertionError, logfun=None, timeout=GLOBAL_TIMEOUT,
+       interval=0.001)
+def call_until(fun, expr):
     """Keep calling function for timeout secs and exit if eval()
     expression is True.
     """
-    stop_at = time.time() + timeout
-    while time.time() < stop_at:
-        ret = fun()
-        if eval(expr):
-            return ret
-        time.sleep(0.001)
-    raise RuntimeError('timed out after %s secs (ret=%r)' % (timeout, ret))
+    ret = fun()
+    assert eval(expr)
+    return ret
 
 
 # ===================================================================
@@ -421,30 +472,31 @@ def call_until(fun, expr, timeout=GLOBAL_TIMEOUT):
 # ===================================================================
 
 
-def safe_remove(file):
-    "Convenience function for removing temporary test files"
+def safe_rmpath(path):
+    "Convenience function for removing temporary test files or dirs"
     try:
-        os.remove(file)
+        st = os.stat(path)
+        if stat.S_ISDIR(st.st_mode):
+            os.rmdir(path)
+        else:
+            os.remove(path)
     except OSError as err:
         if err.errno != errno.ENOENT:
-            # # file is being used by another process
-            # if WINDOWS and isinstance(err, WindowsError) and err.errno == 13:
-            #     return
             raise
 
 
-def safe_rmdir(dir):
-    "Convenience function for removing temporary test directories"
+def safe_mkdir(dir):
+    "Convenience function for creating a directory"
     try:
-        os.rmdir(dir)
+        os.mkdir(dir)
     except OSError as err:
-        if err.errno != errno.ENOENT:
+        if err.errno != errno.EEXIST:
             raise
 
 
 @contextlib.contextmanager
 def chdir(dirname):
-    """Context manager which temporarily changes the current directory."""
+    "Context manager which temporarily changes the current directory."
     curdir = os.getcwd()
     try:
         os.chdir(dirname)
@@ -453,28 +505,60 @@ def chdir(dirname):
         os.chdir(curdir)
 
 
+def create_exe(outpath, c_code=None):
+    """Creates an executable file in the given location."""
+    assert not os.path.exists(outpath), outpath
+    if which("gcc"):
+        if c_code is None:
+            c_code = textwrap.dedent(
+                """
+                #include <unistd.h>
+                int main() {
+                    pause();
+                    return 1;
+                }
+                """)
+        with tempfile.NamedTemporaryFile(
+                suffix='.c', delete=False, mode='wt') as f:
+            f.write(c_code)
+        try:
+            subprocess.check_call(["gcc", f.name, "-o", outpath])
+        finally:
+            safe_rmpath(f.name)
+    else:
+        # fallback - use python's executable
+        if c_code is not None:
+            raise ValueError(
+                "can't specify c_code arg as gcc is not installed")
+        shutil.copyfile(sys.executable, outpath)
+        if POSIX:
+            st = os.stat(outpath)
+            os.chmod(outpath, st.st_mode | stat.S_IEXEC)
+
+
 # ===================================================================
 # --- testing
 # ===================================================================
 
 
-def retry_before_failing(ntimes=None):
+class TestCase(unittest.TestCase):
+
+    def __str__(self):
+        return "%s.%s.%s" % (
+            self.__class__.__module__, self.__class__.__name__,
+            self._testMethodName)
+
+
+# Hack that overrides default unittest.TestCase in order to print
+# a full path representation of the single unit tests being run.
+unittest.TestCase = TestCase
+
+
+def retry_before_failing(retries=NO_RETRIES):
     """Decorator which runs a test function and retries N times before
     actually failing.
     """
-    def decorator(fun):
-        @functools.wraps(fun)
-        def wrapper(*args, **kwargs):
-            times = ntimes or NO_RETRIES
-            assert times, times
-            for x in range(times):
-                try:
-                    return fun(*args, **kwargs)
-                except AssertionError as _:
-                    err = _
-            raise err
-        return wrapper
-    return decorator
+    return retry(exception=AssertionError, timeout=None, retries=retries)
 
 
 def run_test_module_by_name(name):
@@ -535,16 +619,14 @@ def check_net_address(addr, family):
         assert len(octs) == 4, addr
         for num in octs:
             assert 0 <= num <= 255, addr
-        if ipaddress:
-            if not PY3:
-                addr = unicode(addr)
-            ipaddress.IPv4Address(addr)
+        if not PY3:
+            addr = unicode(addr)
+        ipaddress.IPv4Address(addr)
     elif family == AF_INET6:
         assert isinstance(addr, str), addr
-        if ipaddress:
-            if not PY3:
-                addr = unicode(addr)
-            ipaddress.IPv6Address(addr)
+        if not PY3:
+            addr = unicode(addr)
+        ipaddress.IPv6Address(addr)
     elif family == psutil.AF_LINK:
         assert re.match('([a-fA-F0-9]{2}[:|\-]?){6}', addr) is not None, addr
     else:
@@ -611,48 +693,16 @@ def check_connection_ntuple(conn):
                     assert dupsock.type == conn.type
 
 
-def create_temp_executable_file(suffix, c_code=None):
-    tmpdir = None
-    if TRAVIS and OSX:
-        tmpdir = "/private/tmp"
-    fd, path = tempfile.mkstemp(
-        prefix='psu', suffix=suffix, dir=tmpdir)
-    os.close(fd)
-
-    if which("gcc"):
-        if c_code is None:
-            c_code = textwrap.dedent(
-                """
-                #include <unistd.h>
-                void main() {
-                    pause();
-                }
-                """)
-        fd, c_file = tempfile.mkstemp(
-            prefix='psu', suffix='.c', dir=tmpdir)
-        os.close(fd)
-        with open(c_file, "w") as f:
-            f.write(c_code)
-        subprocess.check_call(["gcc", c_file, "-o", path])
-        safe_remove(c_file)
-    else:
-        # fallback - use python's executable
-        shutil.copyfile(sys.executable, path)
-        if POSIX:
-            st = os.stat(path)
-            os.chmod(path, st.st_mode | stat.S_IEXEC)
-    return path
-
-
 def cleanup():
-    reap_children(recursive=True)
-    safe_remove(TESTFN)
-    try:
-        safe_rmdir(TESTFN_UNICODE)
-    except UnicodeEncodeError:
-        pass
+    for name in os.listdir('.'):
+        if name.startswith(TESTFILE_PREFIX):
+            try:
+                safe_rmpath(name)
+            except UnicodeEncodeError as exc:
+                warn(exc)
     for path in _testfiles:
-        safe_remove(path)
+        safe_rmpath(path)
+
 
 atexit.register(cleanup)
 atexit.register(lambda: DEVNULL.close())
