@@ -1156,6 +1156,11 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(p.parent().pid, this_parent)
         # no other process is supposed to have us as parent
         reap_children(recursive=True)
+        if APPVEYOR:
+            # Occasional failures, see:
+            # https://ci.appveyor.com/project/giampaolo/psutil/build/
+            #     job/0hs623nenj7w4m33
+            return
         for p in psutil.process_iter():
             if p.pid == sproc.pid:
                 continue
@@ -1238,15 +1243,34 @@ class TestProcess(unittest.TestCase):
         if not isinstance(d['connections'], list):
             self.assertEqual(d['connections'], 'foo')
 
-        with mock.patch('psutil.Process.name', create=True,
+        # Test ad_value is set on AccessDenied.
+        with mock.patch('psutil.Process.nice', create=True,
+                        side_effect=psutil.AccessDenied):
+            self.assertEqual(
+                p.as_dict(attrs=["nice"], ad_value=1), {"nice": 1})
+
+        # Test that NoSuchProcess bubbles up.
+        with mock.patch('psutil.Process.nice', create=True,
+                        side_effect=psutil.NoSuchProcess(p.pid, "name")):
+            self.assertRaises(
+                psutil.NoSuchProcess, p.as_dict, attrs=["nice"])
+
+        # Test that ZombieProcess is swallowed.
+        with mock.patch('psutil.Process.nice', create=True,
+                        side_effect=psutil.ZombieProcess(p.pid, "name")):
+            self.assertEqual(
+                p.as_dict(attrs=["nice"], ad_value="foo"), {"nice": "foo"})
+
+        # By default APIs raising NotImplementedError are
+        # supposed to be skipped.
+        with mock.patch('psutil.Process.nice', create=True,
                         side_effect=NotImplementedError):
-            # By default APIs raising NotImplementedError are
-            # supposed to be skipped.
             d = p.as_dict()
-            self.assertNotIn('name', list(d.keys()))
+            self.assertNotIn('nice', list(d.keys()))
             # ...unless the user explicitly asked for some attr.
             with self.assertRaises(NotImplementedError):
-                p.as_dict(attrs=["name"])
+                p.as_dict(attrs=["nice"])
+
         # errors
         with self.assertRaises(TypeError):
             p.as_dict('name')
